<?php

namespace App\Models;

use App\Traits\HasProductTagsTrait;
use Illuminate\Database\Eloquent\Builder;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use PhpMyAdmin\SqlParser\Utils\Table;
use Illuminate\Database\Eloquent\SoftDeletes;

class Product extends Model
{
    use HasFactory, HasProductTagsTrait;

    /**
     * Define `stock_status`
     *
     * @var mixed
     */
    CONST STATUS_OUT_OF_STOCK = 1;
    CONST STATUS_LOW_STOCK = 2;
    CONST STATUS_AVAILABLE_STOCK = 3;
    CONST STATUS_NOT_AVAILABLE = 4;

    /**
     * Mass fillable fields
     *
     * @var array
     */
    protected $fillable = [
        'size_id',
        'image',
        'price'
    ];

    /**
     * Attributes should cast
     *
     * @var array
     */
    protected $casts = [
        // 'price' => 'integer',
        'pack' => 'integer'
    ];

    /**
     * Append custom attributes
     *
     * @var array
     */
    protected $appends = [
        'image_url'
    ];


      /**
     * Relationship to `model` table
     *
     * @return mixed
     */
    public function brand_details(){
        return $this->hasOne(Brand::class,'id', 'brand_id')->withDefault(['name' => '']);
    }
    
    /**
     * Relationship to `model` table
     *
     * @return mixed
     */
    public function model_details(){
        return $this->hasOne(Models::class,'id', 'model_id')->withDefault(['name' => '']);
    }

    /**
     * Relationship to `categories` table
     *
     * @return mixed
     */
    public function category()
    {
        return $this->belongsTo(Category::class)->withDefault(['cat_name' => '']);
    }

        /**
     * Relationship to `categories` table
     *
     * @return mixed
     */
    public function subcategory()
    {
        return $this->belongsTo(Category::class,'sub_category_id', 'id')->withDefault(['cat_name' => '']);
    }




    /**
     * Relationship to `suppliers` table
     *
     * @return mixed
     */
    public function supplier()
    {
        return $this->belongsTo(Supplier::class)->withDefault(['supplier_name' => '']);
    }

    /**
     * Relationship to `product tags` table
     *
     * @return mixed
     */
    public function productTags()
    {
        return $this->belongsToMany(ProductTag::class, 'product_has_tags', 'product_id', 'product_tag_id');
    }

    /**
     * Relationship to `product_main_stocks` table
     *
     * @return mixed
     */
    public function getQuantity(){
        return $this->hasMany(StockLog::class)->withDefault(['quantity' => 0]);
    }


    /**
     * Relationship to `order_purchase_details` table
     *
     * @return mixed
     */
    public function getIncoming(){
        return $this->hasMany(OrderPurchaseDetail::class, 'product_code', 'product_code');
    }




     /**
     * Relationship to `product_costs_default_supplier` table
     *
     * @return mixed
     */
    public function productCostDetails(){
        return $this->hasOne(ProductCost::class,'product_id', 'id')->withDefault(['default_supplier' => 1]);
    }




    /**
     * Relationship to `product_costs` table WHERE default supplier = 1
     *
     * @return mixed
     */
    public function preferredProductCost(){
        return $this->hasOne(ProductCost::class, 'product_id', 'id')->where('default_supplier', '=', 1)->withDefault(['cost' => 0]);
    }

    /**
     * Relationship to `users` table
     *
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     */
    public function seller()
    {
        return $this->belongsTo(User::class, 'shop_id', 'id');
    }

    /**
     * Relationship to `product_costs` table
     *
     * @return mixed
     */
    public function productCost(){
        return $this->hasMany(ProductCost::class, 'product_id', 'id')->withDefault(['cost' => 0]);
    }

    /**
     * Relationship to `order_purchase_details` table
     *
     * @return mixed
     */
    public function getProductPrice(){
        return $this->hasMany(ProductPrice::class);
    }


    /**
     * Query to search by name from table
     *
     * @param \Illuminate\Database\Query\Builder $query
     * @param string|null $keyword
     * @return \Illuminate\Database\Query\Builder
     */
    public function scopeSearchTable($query, $keyword = null)
    {
        if (!empty($keyword)) {
            $productTable = $this->getTable();
            $modelTable = (new Models())->getTable();

            return $query->where(function(Builder $product) use ($productTable, $modelTable,$keyword) {
 
                $product->where("{$productTable}.chassis_no", 'like', "%$keyword%")
                ->orWhere("{$productTable}.engine_no", 'like', "%$keyword%")
                ->orWhere("{$modelTable}.name", 'like', "%$keyword%");                    
            });
        }
        return;
    }

    /**
     * Query to search by name from table
     *
     * @param \Illuminate\Database\Query\Builder $query
     * @param string|null $keyword
     * @return \Illuminate\Database\Query\Builder
     */
    public function scopeSearchModel($query, $keyword = null)
    {
        

        return NULL;
    }
    

    /**
     * Sub Query to get total of incoming / order_purchase_detail
     *
     * @param \Illuminate\Database\Query\Builder $query
     * @return \Illuminate\Database\Query\Builder
     */
    public function scopeTotalIncoming($query)
    {
        $productTable = $this->getTable();
        $orderPurchaseTable = (new OrderPurchase())->getTable();
        $orderPurchaseDetailsTable = (new OrderPurchaseDetail())->getTable();

        return $query->addSelect(['total_incoming' => OrderPurchaseDetail::selectRaw("
        SUM({$orderPurchaseDetailsTable}.quantity)
        ")
        ->join("{$orderPurchaseTable}", "{$orderPurchaseTable}.id", '=', "{$orderPurchaseDetailsTable}.purchase_id")
            ->whereColumn("{$orderPurchaseDetailsTable}.product_code", "{$productTable}.product_code")
            ->where("{$orderPurchaseTable}.status", '<>', OrderPurchase::STATUS_CLOSE)
            ->limit(1)
        ]);
    }


    /**
     * Sub Query to get the quantity
     *
     * @param \Illuminate\Database\Query\Builder $query
     * @return \Illuminate\Database\Query\Builder
     */
    public function scopeQuantity($query)
    {
        return $query->addSelect(['check_in' => StockLog::selectRaw("SUM(quantity)")
        ->whereColumn('stock_logs.product_id', 'products.id')
        ->where('stock_logs.stock_status', 1)        
        ])
        ->addSelect(['check_out' => StockLog::selectRaw("SUM(quantity)")
        ->whereColumn('stock_logs.product_id', 'products.id')
        ->where('stock_logs.stock_status', 0)        
        ])
        ->addSelect(['defect' => StockLog::selectRaw("SUM(quantity)")
        ->whereColumn('stock_logs.product_id', 'products.id')
        ->where('stock_logs.stock_status', 2)        
        ])
        ->addSelect(['sales_quantity' => OrderManagementDetail::selectRaw("SUM(quantity)")
                ->whereColumn('order_management_details.product_id', 'products.id')
        ]);
    }


    /**
     * Query to filter products by category
     *
     * @param \Illuminate\Database\Query\Builder $query
     * @param null $categoryId
     * @return \Illuminate\Database\Query\Builder
     */
    public function scopeFilterByCategory($query, $categoryId = null)
    {
        if (!empty($categoryId)) {
            return $query->where('category_id', $categoryId);
        }

        return;
    }

        /**
     * Query to filter products by category
     *
     * @param \Illuminate\Database\Query\Builder $query
     * @param null $categoryId
     * @return \Illuminate\Database\Query\Builder
     */
    public function scopeSubCategory($query, $categoryId = null)
    {
        if (!empty($categoryId)) {
            return $query->where('category_id1', $categoryId);
        }

        return;
    }

    /**
     * Query to filter products by product tags
     *
     * @param \Illuminate\Database\Query\Builder $query
     * @param null $productTag
     * @return \Illuminate\Database\Query\Builder
     */
    public function scopeFilterByProductTag($query, $productTag = null)
    {
        if (!empty($productTag)) {
            $productTagsTable = (new ProductTag())->getTable();

            return $query->whereHas('productTags', function ($query) use ($productTagsTable, $productTag) {
                 $query->where("{$productTagsTable}.id", $productTag);
            });
        }

        return;
    }

    /**
     * Query to filter products by Auth user
     *
     * @param \Illuminate\Database\Query\Builder $query
     * @param $userRole
     * @param null $userId
     * @return \Illuminate\Database\Query\Builder
     */
    public function scopeFilterByAuthUser($query, $userRole, $userId = null)
    {
        // if (!empty($userRole)) {
        //     if ($userRole == 'dropshipper'){
        //         $userPermissions = Permission::dropshipperProductPermissions($userId);
        //         return $query->whereIn('id', $userPermissions);
        //     } else

        //         return;
        // }

        return;
    }

    /**
     * Update product cost price and cost currency
     *
     * @param int $id
     * @param array $data
     */
    public static function updateData($id,$data = []){
        DB::table('products')->where('id', $id)->update($data);
    }

    /**
     * Accessor for `image_url`
     *
     * @return string
     */
    public function getImageUrlAttribute()
    {
        if (!empty($this->attributes['image']) && file_exists(public_path($this->attributes['image']))) {
            return asset($this->attributes['image']);
        }

        return asset('No-Image-Found.png');
    }





    /**
     * Get All `products` By Seller ID
     * param int
     * @return mixed
     */
    public static function getProductsBySellerID($shopId){
        return DB::table('products')->where('shop_id', $shopId)->get();
   }

   public function scopeSearchSelectTwo($query, $keyword = null)
    {
        if (!empty($keyword)) {
            return $query->where(function($query) use ($keyword) {
                $query->where('part_name', 'like', "%$keyword%");
                $query->orWhere('id', 'like', "%$keyword%");
            });
        }

        return;
    }
}
